# IEDMod release 3 by David Bowland
# ./addons/eventscripts/iedmod/iedmod.py

# >>> To configure this addon please see iedmod.cfg (created when the addon is first loaded) <<<

"""
Allows players to create props which explode when players are near. Requires only ES 2.0.0.248+
"""


import cfglib
import es
import gamethread
import langlib
import os.path
import playerlib
import random
import time

import psyco
psyco.full()


info = es.AddonInfo()
info.name     = 'IEDMod'
info.version  = '3'
info.basename = 'iedmod'
info.url      = 'http://addons.eventscripts.com/addons/view/' + info.basename
info.author   = 'SuperDave'


###


config = cfglib.AddonCFG(es.getAddonPath(info.basename) + '/' + info.basename + '.cfg')

config.text(info.name + ' release %s options' % info.version)
config.text('./addons/eventscripts/%(basename)s/%(basename)s.cfg' % {'basename':info.basename})
config.text(info.url)
config.text('')
config.text('Load this addon with: es_load ' + info.basename)
config.text('\n')
config.text('To configure language settings for %s please see %s_languages.ini' % (info.name, info.basename))
config.text('\n')

config.text('***** Detonation options *****')
cvar_detonation_radius     = config.cvar('iedmod_detonation_radius',     150, 'IEDs will be detonated when a player is within this distance')
cvar_detonation_checkdelay = config.cvar('iedmod_detonation_checkdelay', 1,   'Number of seconds between checking the distance of players')
cvar_detonation_team       = config.cvar('iedmod_detonation_team',       0,   '0 = only opposing team members can detonate an IED, 1 = any player can detonate an IED')
config.text('\n')

config.text('***** Explosion options *****')
cvar_explosion_radius    = config.cvar('iedmod_explosion_radius',    350, 'Radius of IED explosion--HE grenade is 350')
cvar_explosion_magnitude = config.cvar('iedmod_explosion_magnitude', 100, 'Magnitude of IED explosion--HE grenade is 100')
cvar_explosion_sound     = config.cvar('iedmod_explosion_sound',     40,  'Percent of the map that can hear an IED explode')
config.text('\n')

config.text('***** IED purchasing options *****')
cvar_purchase_cost           = config.cvar('iedmod_purchase_cost',           2000,      'Cost of purchasing an IED--0 will give all players the maximum number of IEDs')
cvar_purchase_max            = config.cvar('iedmod_purchase_max',            3,         'Maximum number of IEDs a player can have--set to 0 for no limit')
cvar_purchase_command        = config.cvar('iedmod_purchase_command',        '!buyied', 'Command players must say to purchase an IED')
cvar_purchase_buyzone        = config.cvar('iedmod_purchase_buyzone',        1,         '0 = IEDs can be puchased anywhere, 1 = IEDs can only be puchased in buy zones')
cvar_purchase_loseafterdeath = config.cvar('iedmod_purchase_loseafterdeath', 1,         '0 = players keep IEDs that have not been planted when killed, 1 = players will lose IEDs that have not been planted when killed')
config.text('\n')

config.text('***** IED planting options *****')
config.text('IEDs will not start checking for near players until the player who created the IED moves out of the detonation radius.')
cvar_create_command = config.cvar('iedmod_create_command', '!ied', 'Command players must say to plant an IED')
cvar_create_delay   = config.cvar('iedmod_create_delay',   5,      'Number of seconds a player must wait after creating an IED to create another--set to 0 for no delay')
cvar_create_knife   = config.cvar('iedmod_create_knife',   1,      '0 = players can create an IED with any active weapon, 1 = players are required to have the knife as active weapon to create an IED, 2 = players will be forced to use the knife when creating an IED')
cvar_create_sound   = config.cvar('iedmod_create_sound',   40,     'Percent of the map that can hear an IED beep when activated--set to 0 for no sound')
config.text('\n')

config.text('***** Bot options *****')
config.text('Bots will only purchase an IED if they have the cost of an IED plus bot_eco_limit.')
cvar_bot_allowied           = config.cvar('iedmod_bot_allowied',           2,  'Number of IEDs bot will be allowed to buy or plant per round')
cvar_bot_purchasechance     = config.cvar('iedmod_bot_purchasechance',     80, 'Percent chance a bot will purchase an IED if it has enough money')
cvar_bot_createdelay_min    = config.cvar('iedmod_bot_createdelay_min',    5,  'Minumim number of seconds after spawning a bot will wait to create an IED')
cvar_bot_createdelay_max    = config.cvar('iedmod_bot_createdelay_max',    45, 'Maximum number of seconds after spawning a bot will wait to create an IED')
cvar_bot_iterationstoremove = config.cvar('iedmod_bot_iterationstoremove', 4,  'Number of IED distance updates until an IED spawned by a bot is removed if that bot is stuck in the IED')
config.text('\n')

config.text('***** Announcement options *****')
cvar_announce_spawn = config.cvar('iedmod_announce_spawn', 1, '0 = no change, 1 = announce to each player his or her IED count as well as how to purchase and create IEDs when the player spawns')
config.text('')
config.cvar('iedmod_announce_purchase_chat',   0, '0 = no change, 1 = announce IED purchasing feedback in chat area')
config.cvar('iedmod_announce_purchase_center', 1, '0 = no change, 1 = announce IED purchasing feedback in center text')
config.cvar('iedmod_announce_purchase_hud',    1, '0 = no change, 1 = announce IED purchasing feedback in a HUD message')
config.text('')
config.cvar('iedmod_announce_create_chat',   0, '0 = no change, 1 = announce IED creation feedback in chat area')
config.cvar('iedmod_announce_create_center', 1, '0 = no change, 1 = announce IED creation feedback in center text')
config.cvar('iedmod_announce_create_hud',    1, '0 = no change, 1 = announce IED creation feedback in a HUD message')
config.text('\n')

config.text('***** IED model options *****')
config.text('NOTE: IEDs will be spawned as prop_physics entities. Some will move when shot and others will be destroyed.')
config.text('Destroyed models are no longer IEDs.')
config.text('')
config.text('IED models can be added with the following server command:')
config.text('')
config.text('iedmod_addmodel <"model relative to ./models/">')
config.text('')
config.text('Models are marked for download automatically.')
config.text('\n')

config.text('Place your iedmod_addmodel commands below:')
config.command('iedmod_addmodel')
config.text('\n')

config.text('Uncomment the following default options if you wish to use them:')
config.text('')
config.text('iedmod_addmodel props_c17/oildrum001.mdl')
config.text('iedmod_addmodel props_c17/oildrum001_explosive.mdl')
config.text('iedmod_addmodel props_junk/cardboard_box001a.mdl')
config.text('iedmod_addmodel props_junk/cardboard_box001b.mdl')
config.text('iedmod_addmodel props_junk/cardboard_box002a.mdl')
config.text('iedmod_addmodel props_junk/cardboard_box002b.mdl')
config.text('iedmod_addmodel props_junk/cardboard_box003a.mdl')
config.text('iedmod_addmodel props_junk/cardboard_box003b.mdl')
config.text('iedmod_addmodel props_junk/cardboard_box004a.mdl')
config.text('iedmod_addmodel props_junk/garbage_bag001a.mdl')
config.text('iedmod_addmodel props_junk/gascan001a.mdl')
config.text('iedmod_addmodel props_junk/metal_paintcan001a.mdl')
config.text('iedmod_addmodel props_junk/metal_paintcan001b.mdl')
config.text('iedmod_addmodel props_junk/MetalBucket01a.mdl')
config.text('iedmod_addmodel props_junk/metalgascan.mdl')
config.text('iedmod_addmodel props_junk/plasticbucket001a.mdl')
config.text('iedmod_addmodel props_junk/propane_tank001a.mdl')
config.text('iedmod_addmodel props_junk/PropaneCanister001a.mdl')
config.text('iedmod_addmodel props_junk/TrafficCone001a.mdl')
config.text('iedmod_addmodel props_junk/TrashBin01a.mdl')
config.text('iedmod_addmodel props_junk/watermelon01.mdl')
config.text('iedmod_addmodel props_junk/wood_crate001a.mdl')
config.text('iedmod_addmodel props_junk/wood_crate002a.mdl')
config.text('\n')

config.text('IED models can be removed with the following server command:')
config.text('')
config.text('iedmod_removemodel <"model relative to ./models/">')
config.text('')
config.text('Due to the fact IED models can be added and removed above, this command will not be used by most users.')
config.text('\n')

config.text('***** IED sound options *****')
config.text('')
config.text('IED sounds can be added with the following server command:')
config.text('')
config.text('iedmod_addsound <"sound relative to ./sound/">')
config.text('')
config.text('Sounds are marked for download automatically.')
config.text('\n')

config.text('Place your iedmod_addsound commands below:')
config.command('iedmod_addsound')
config.text('\n')

config.text('Uncomment the following default options if you wish to use them:')
config.text('')
config.text('iedmod_addsound buttons/blip1.wav')
config.text('iedmod_addsound buttons/button14.wav')
config.text('iedmod_addsound buttons/button15.wav')
config.text('iedmod_addsound buttons/button16.wav')
config.text('iedmod_addsound buttons/button17.wav')
config.text('iedmod_addsound buttons/button18.wav')
config.text('iedmod_addsound buttons/button24.wav')
config.text('iedmod_addsound buttons/button3.wav')
config.text('iedmod_addsound buttons/button9.wav')
config.text('iedmod_addsound npc/roller/remote_yes.wav')
config.text('iedmod_addsound npc/turret_floor/ping.wav')
config.text('\n')

config.text('IED sounds can be removed with the following server command:')
config.text('')
config.text('iedmod_removesound <"sound relative to ./sound/">')
config.text('')
config.text('Due to the fact IED sounds can be added and removed above, this command will not be used by most users.')
config.text('\n')

config.text('***** Event documentation *****')
config.text('')
config.text('The following events will fire while IEDMod is loaded:')
config.text('')
config.text('ied_activated')
config.text('- An IED is "activated" when it begins checking for near players.')
config.text('   event_var(userid)          - Player whose IED was activated')
config.text('   event_var(index)           - Index of IED')
config.text('')
config.text('ied_exploded')
config.text('   event_var(userid)          - Player who triggered the IED')
config.text('   event_var(attacker)        - Player whose IED exploded')
config.text('   event_var(index)           - Index of IED that exploded (WARNING: Entity no longer exists when event fires)')
config.text('')
config.text('ied_planted')
config.text('   event_var(userid)          - Player who planted the IED')
config.text('   event_var(index)           - Index of IED')
config.text('')
config.text('ied_purchased')
config.text('   event_var(userid)          - Player who purchased an IED')
config.text('   event_var(count)           - Number of IEDs the player owns')
config.text('')
config.text('Each event also provides event_var(es_steamid), event_var(es_username), and all the other event_vars that accompany')
config.text('event_var(userid). More information can be found here: http://www.eventscripts.com/pages/Extended_event_variables')

config.write()

lastgive = es.ServerVar('eventscripts_lastgive')


###


class IEDMaterial(object):
   itempath = es.ServerVar('eventscripts_gamedir') + '/'
   lastitem = ''

   def __init__(self, shortpath):
      self.shortpath = shortpath + '/'

      self.items = set()
      self.itempath += self.shortpath

   def addItem(self, item):
      self.items.add(item)

   def removeItem(self, item):
      if item in self.items:
         self.items.remove(item)

   def getItem(self):
      if not self.items: return None

      choices       = filter(lambda x: x <> self.lastitem, self.items)
      self.lastitem = random.choice(choices if choices else list(self.items))

      return self.lastitem

   def markItemsForDownload(self):
      for item in filter(lambda x: os.path.isfile(self.itempath + x), self.items):
         es.stringtable('downloadables', self.shortpath + item)


sounds = IEDMaterial('sound')

def addsound_cmd():
   """
   iedmod_addsound <"sound relative to ./sound/">
   """
   if es.getargc() == 2:
      sounds.addItem(es.getargv(1))

   else:
      es.dbgmsg(0, 'Syntax: iedmod_addsound <\"sound relative to ./sound/\">')

if not es.exists('command', 'iedmod_addsound'):
   es.regcmd('iedmod_addsound', 'iedmod/addsound_cmd', 'iedmod_addsound <\"sound relative to ./sound/\">\nServer command for adding IED activation sounds')


def removesound_cmd():
   """
   iedmod_removesound <"sound relative to ./sound/">
   """
   if es.getargc() == 2:
      sounds.removeItem(es.getargv(1))

   else:
      es.dbgmsg(0, 'Syntax: iedmod_removesound <\"sound relative to ./sound/\">')

if not es.exists('command', 'iedmod_removesound'):
   es.regcmd('iedmod_removesound', 'iedmod/removesound_cmd', 'iedmod_removesound <\"sound relative to ./sound/\">\nServer command for removing IED activation sounds')


models = IEDMaterial('models')

def addmodel_cmd():
   """
   iedmod_addmodel <"model relative to ./models/">
   """
   if es.getargc() == 2:
      models.addItem(es.getargv(1))

   else:
      es.dbgmsg(0, 'Syntax: iedmod_addmodel <\"model relative to ./models/\">')

if not es.exists('command', 'iedmod_addmodel'):
   es.regcmd('iedmod_addmodel', 'iedmod/addmodel_cmd', 'iedmod_addmodel <\"model relative to ./models/\">\nServer command for adding IED models')


def removemodel_cmd():
   """
   iedmod_removemodel <"model relative to ./models/">
   """
   if es.getargc() == 2:
      models.removeItem(es.getargv(1))

   else:
      es.dbgmsg(0, 'Syntax: iedmod_removemodel <\"model relative to ./models/\">')

if not es.exists('command', 'iedmod_removemodel'):
   es.regcmd('iedmod_removemodel', 'iedmod/removemodel_cmd', 'iedmod_removemodel <\"model relative to ./models/\">\nServer command for removing IED models')


###


class IEDManager(object):
   class IED(object):
      activating = 1

      def __init__(self, index, userid):
         self.index  = index
         self.userid = userid
         self.team   = es.getplayerteam(userid)

      def __getattr__(self, name):
         if name == 'origin':
            return str(es.getindexprop(self.index, 'CBaseEntity.m_vecOrigin'))

         elif name == 'location':
            return map(float, self.origin.split(','))

         raise AttributeError, 'IED instance has no attribute \'%s\'' % name

      def activate(self):
         if not self.activating: return

         self.activating = 0

         es.event('initialize', 'ied_activated')
         es.event('setint',     'ied_activated', 'userid', self.userid)
         es.event('setint',     'ied_activated', 'index',  self.index)
         es.event('fire',       'ied_activated')

         create_sound = float(cvar_create_sound)
         if create_sound:
            sound = sounds.getItem()
            if sound:
               es.emitsound('entity', self.index, sound, 1, (100. - create_sound) / 100)

      def explode(self):
         for index in es.createentitylist('env_explosion'):
            es.server.cmd('es_xremove %s' % index)

         explode_sound = float(cvar_explosion_sound)
         if explode_sound:
            es.emitsound('entity', self.index, 'weapons/hegrenade/explode%s.wav' % random.randint(3, 5), 1, (100. - explode_sound) / 100)

         es.server.cmd('es_xentcreate %s env_explosion' % self.userid)
         es.setindexprop(lastgive, 'CBaseEntity.m_hOwnerEntity', es.getplayerhandle(self.userid))

         es.server.queuecmd('es_xfire %s env_explosion addoutput \"origin %s\"' % (self.userid, self.origin.replace(',', ' ')))
         es.server.queuecmd('es_xfire %s env_explosion addoutput \"imagnitude %s\"' % (self.userid, cvar_explosion_magnitude))
         es.server.queuecmd('es_xfire %s env_explosion addoutput \"iradiusoverride %s\"' % (self.userid, cvar_explosion_radius))
         es.server.queuecmd('es_xfire %s env_explosion explode' % self.userid)
         es.server.queuecmd('es_xfire %s env_explosion kill' % self.userid)

         es.server.cmd('es_xremove %s' % self.index)
         players.getPlayer(self.userid).removeIED(self.index)

   """ Begin IEDManager """

   def __init__(self):
      self.indexes = {}

   def registerIED(self, index, userid):
      if not self.indexes:
         gamethread.delayedname(cvar_detonation_checkdelay, 'iedmod_update', self.updateIEDs)

      self.indexes[index] = self.IED(index, userid)

      es.event('initialize', 'ied_planted')
      es.event('setint',     'ied_planted', 'userid', userid)
      es.event('setint',     'ied_planted', 'index',  index)
      es.event('fire',       'ied_planted')

   def removeIED(self, index):
      if index not in self.indexes: return

      del self.indexes[index]
      if not self.indexes:
         self.cancelDelay()

      if index in self.physicsProps():
         es.server.cmd('es_xremove %s' % index)

   def clearIEDs(self):
      props = self.physicsProps()
      for index in self.indexes:
         if index in props:
            es.server.cmd('es_xremove %s' % index)

      self.indexes.clear()
      self.cancelDelay()

   def updateIEDs(self):
      removed = []
      props   = self.physicsProps()

      livingplayers = {}
      for userid in filter(lambda x: not players.getPlayer(x).isDead(), es.getUseridList()):
         livingplayers[userid] = es.getplayerteam(userid), es.getplayerlocation(userid)

      team_option     = int(cvar_detonation_team)
      radius_option   = int(cvar_detonation_radius)
      activate_remove = int(cvar_bot_iterationstoremove)

      for index in self.indexes:
         ied = self.indexes[index]
         if index in props:
            ied_location = ied.location
            ied_team     = (2, 3) if team_option or ied.activating else (5 - ied.team,)
            ied_activate = True

            for userid in livingplayers:
               team, location = livingplayers[userid]
               if team in ied_team:
                  if sum((location[x] - ied_location[x]) ** 2 for x in range(3)) ** 0.5 <= radius_option:
                     if ied.activating:
                        ied.activating += 1
                        ied_activate = False

                        if ied.activating > activate_remove:
                           es.server.cmd('es_xremove %s' % index)
                           players.getPlayer(ied.userid).removeIED(index)
                           removed.append(index)

                     else:
                        ied.explode()

                        es.event('initialize', 'ied_exploded')
                        es.event('setint',     'ied_exploded', 'userid',   userid)
                        es.event('setint',     'ied_exploded', 'attacker', ied.userid)
                        es.event('setint',     'ied_exploded', 'index',    index)
                        es.event('fire',       'ied_exploded')

                        removed.append(index)

                     break

            if ied_activate: ied.activate()

         else:
            players.getPlayer(ied.userid).removeIED(index)
            removed.append(index)

      for index in removed:
         del self.indexes[index]

      if self.indexes:
         gamethread.delayedname(cvar_detonation_checkdelay, 'iedmod_update', self.updateIEDs)

   @staticmethod
   def cancelDelay():
      gamethread.cancelDelayed('iedmod_update')

   @staticmethod
   def physicsProps():
      return es.createentitylist('prop_physics')

ieds = IEDManager()


###


class IEDPlayer(object):
   bot_eco_limit = es.ServerVar('bot_eco_limit')

   prevent = 0
   count   = 0

   def __init__(self, userid):
      self.userid  = userid
      self.indexes = set()
      self.is_bot  = es.getplayersteamid(userid) == 'BOT'

   def cancelDelayed(self):
      gamethread.cancelDelayed('iedmod_player_%s' % self.userid)

   def clearIEDs(self):
      for index in self.indexes:
         ieds.removeIED(index)
      self.indexes.clear()

   def playerSpawn(self):
      self.cancelDelayed()
      self.prevent = 0

      cost = int(cvar_purchase_cost)
      if not cost:
         self.count = int(cvar_purchase_max)

      self.spawnAnnounce()
      if not self.is_bot: return

      bot_allow_ied = int(cvar_bot_allowied)
      if bot_allow_ied:
         if cost:
            eco_limit  = int(self.bot_eco_limit)
            buy_chance = int(cvar_bot_purchasechance)

            for int_x in xrange(self.count, bot_allow_ied):
               if random.randint(1, 100) > buy_chance or self.getCash() < cost + eco_limit: break
               self.buyIED()

         else:
            self.count = bot_allow_ied

      self.botCreateIED()

   def playerDeath(self):
      self.cancelDelayed()

      if int(cvar_purchase_loseafterdeath):
         self.count = 0

   def buyIED(self):
      if self.isDead():
         self.announceType('purchase', 'purchase dead')
         return

      if not self.inBuyZone():
         self.announceType('purchase', 'purchase buy zone')
         return

      ied_max = int(cvar_purchase_max)
      cost    = int(cvar_purchase_cost)
      if (self.count >= ied_max and ied_max) or not cost:
         self.announceType('purchase', 'purchase max')
         return

      cash = self.getCash()
      if cost > cash:
         self.announceType('purchase', 'purchase cash', {'cost':cost})
         return

      self.setCash(cash - cost)
      self.count += 1

      es.event('initialize', 'ied_purchased')
      es.event('setint',     'ied_purchased', 'userid', self.userid)
      es.event('setint',     'ied_purchased', 'count',  self.count)
      es.event('fire',       'ied_purchased')

      if self.announceType('purchase', 'purchase success'):
         self.spawnAnnounce()

   def createIED(self):
      if self.isDead():
         self.announceType('create', 'create dead')
         return

      if self.prevent > time.time():
         self.announceType('create', 'create limit')
         return

      if not self.count and int(cvar_purchase_max):
         self.announceType('create', 'create empty')
         return

      require_knife = int(cvar_create_knife)
      if require_knife == 2 or (require_knife == 1 and self.is_bot):
         es.sexec(self.userid, 'use weapon_knife')
         require_knife = 0
      if es.createplayerlist(self.userid)[self.userid]['weapon'] <> 'weapon_knife' and require_knife:
         self.announceType('create', 'create knife')
         return

      if self.count: self.count -= 1

      model = models.getItem()
      if not model: return
      es.server.cmd('es_xprop_physics_create %s %s' % (self.userid, model))

      index = int(lastgive)
      self.indexes.add(index)
      ieds.registerIED(index, self.userid)

      if self.is_bot:
         self.botCreateIED()

      else:
         self.prevent = time.time() + cvar_create_delay

      self.announceType('create', 'create success')
      if int(cvar_announce_spawn):
         purchase_max = int(cvar_purchase_max)
         if purchase_max:
            es.tell(self.userid, '#multi', lang_text('ied count', {'count':self.count, 'max':purchase_max}, self.getLang()))

         else:
            es.tell(self.userid, '#multi', lang_text('ied count no max', {'count':self.count}, self.getLang()))

   def botCreateIED(self):
      if self.count or not int(cvar_purchase_max):
         gamethread.delayedname(random.randint(max(cvar_bot_createdelay_min, cvar_create_delay), cvar_bot_createdelay_max), 'iedmod_player_%s' % self.userid, self.createIED)

   def removeIED(self, index):
      if index in self.indexes:
         self.indexes.remove(index)

   def isDead(self):
      return es.getplayerprop(self.userid, 'CBasePlayer.pl.deadflag')

   def inBuyZone(self):
      return es.getplayerprop(self.userid, 'CCSPlayer.m_bInBuyZone') % 2 or es.getentityindex('func_buyzone') == -1 or not int(cvar_purchase_buyzone)

   def getLang(self):
      return playerlib.getPlayer(self.userid).get('lang')

   def getCash(self):
      return es.getplayerprop(self.userid, 'CCSPlayer.m_iAccount')

   def setCash(self, amount):
      es.setplayerprop(self.userid, 'CCSPlayer.m_iAccount', amount)

   def hudHint(self, message):
      es.usermsg('create', 'iedmod_hud', 'HintText')
      es.usermsg('write', 'short', 'iedmod_hud', -1)
      es.usermsg('write', 'string', 'iedmod_hud', message)
      es.usermsg('send', 'iedmod_hud', self.userid)
      es.usermsg('delete', 'iedmod_hud')

   def spawnAnnounce(self):
      if int(cvar_announce_spawn):
         player_lang = self.getLang()

         ied_max = int(cvar_purchase_max)
         if ied_max:
            es.tell(self.userid, '#multi', lang_text('ied count', {'count':self.count, 'max':ied_max}, player_lang))

         else:
            es.tell(self.userid, '#multi', lang_text('ied count no max', {'count':self.count}, player_lang))

         cost   = int(cvar_purchase_cost)
         tokens = {'purchase':say_command_buy, 'create':say_command_create, 'cost':cost}
         if (self.count < ied_max or not ied_max) and cost:
            if self.count or not ied_max:
               es.tell(self.userid, '#multi', lang_text('ied create or purchase', tokens, player_lang))

            else:
               es.tell(self.userid, '#multi', lang_text('ied purchase', tokens, player_lang))

         else:
            es.tell(self.userid, '#multi', lang_text('ied create', tokens, player_lang))

   def announceType(self, announce_type, text, tokens={}):
      """Returns: False = no text was displayed, True = at least one announce method was used to display text"""
      displayed   = False
      player_lang = self.getLang()

      if int(es.ServerVar('iedmod_announce_%s_chat' % announce_type)):
         es.tell(self.userid, '#multi', lang_text(text, tokens, player_lang))
         displayed = True

      if int(es.ServerVar('iedmod_announce_%s_center' % announce_type)):
         es.centertell(self.userid, self._removeTags(lang_text(text, tokens, player_lang)))
         displayed = True

      if int(es.ServerVar('iedmod_announce_%s_hud' % announce_type)):
         self.hudHint(self._removeTags(lang_text(text, tokens, player_lang)))
         displayed = True

      return displayed

   @staticmethod
   def _removeTags(text):
      """Removes #lightgreen, #green and #default tags from the supplied string"""
      return text.replace('#lightgreen', '').replace('#green', '').replace('#default', '')


class PlayerManager(object):
   def __init__(self):
      self.players = {}

   def getPlayer(self, userid):
      userid = int(userid)
      if userid not in self.players:
         self.players[userid] = IEDPlayer(userid)

      return self.players[userid]

   def removePlayer(self, userid):
      userid = int(userid)
      if userid in self.players:
         self.players[userid].cancelDelayed()
         self.players[userid].clearIEDs()

         del self.players[userid]

   def clearPlayers(self):
      for userid in self.players:
         self.players[userid].cancelDelayed()
      self.players.clear()

players = PlayerManager()
   

def buy_cmd():
   players.getPlayer(es.getcmduserid()).buyIED()


def create_cmd():
   players.getPlayer(es.getcmduserid()).createIED()


###


def getLangText():
   lang_path = es.getAddonPath(info.basename) + '/%s_languages.ini' % info.basename
   if os.path.isfile(lang_path):
      return langlib.Strings(lang_path)

   else:
      es.dbgmsg(0, '%(name)s: Unable to load %(basename)s_languages.ini! Please ensure it is in the ./%(basename)s/ directory.' % {'name':info.name, 'basename':info.basename})
      return lambda x, y, z: '%s: %s_languages.ini not found' % (info.name, info.basename)

lang_text = getLangText()


###


say_command_buy    = None
say_command_create = None


def load():
   global say_command_buy
   global say_command_create

   config.execute()

   tempsay = str(cvar_purchase_command)
   if tempsay and tempsay.lower() <> 'none':
      if not es.exists('saycommand', tempsay):
         es.regsaycmd(tempsay, 'iedmod/buy_cmd', 'Say command for purchasing an IED')
         say_command_buy = tempsay

   tempsay = str(cvar_create_command)
   if tempsay and tempsay.lower() <> 'none':
      if not es.exists('saycommand', tempsay):
         es.regsaycmd(tempsay, 'iedmod/create_cmd', 'Say command for creating an IED')
         say_command_create = tempsay

   load_events()


def es_map_start(event_var):
   load_events()

   sounds.markItemsForDownload()
   models.markItemsForDownload()

   ieds.clearIEDs()
   players.clearPlayers()


def round_start(event_var):
   ieds.clearIEDs()


def player_spawn(event_var):
   if int(event_var['es_userteam']) > 1:
      players.getPlayer(event_var['userid']).playerSpawn()


def player_death(event_var):
   players.getPlayer(event_var['userid']).playerDeath()


def player_disconnect(event_var):
   players.removePlayer(event_var['userid'])


def unload():
   ieds.clearIEDs()
   players.clearPlayers()

   if say_command_buy:
      es.unregsaycmd(say_command_buy)
   if say_command_create:
      es.unregsaycmd(say_command_create)

   for cvar in config.getCvars():
      es.ServerVar(cvar).set(0)


###


def load_events():
   """Loads the .res file for the events"""
   if os.path.isfile(es.getAddonPath('iedmod') + '/iedmod.res'):
      es.loadevents('declare', 'addons/eventscripts/iedmod/iedmod.res')

   else:
      events = {'ied_activated':{'userid':'short', 'index':'short'}, 'ied_exploded':{'userid':'short', 'attacker':'short', 'index':'short'}, 'ied_planted':{'userid':'short', 'index':'short'}, 'ied_purchased':{'userid':'short', 'count':'short'}}

      if not os.path.isfile(es.getAddonPath('iedmod') + 'es_iedmod_events_db.txt'):
         es.keygroupcreate('iedmod_events')
         for event_name in events:
            es.keycreate('iedmod_events', event_name)
            for event_var in events[event_name]:
               es.keysetvalue('iedmod_events', event_name, event_var, events[event_name][event_var])
         es.keygroupsave('iedmod_events', '|iedmod')
         es.keygroupdelete('iedmod_events')
      es.loadevents('declare', 'addons/eventscripts/iedmod/es_iedmod_events_db.txt')
      es.dbgmsg(0, 'IEDMod: Unable to load iedmod.res! Please ensure it is in the ./iedmod/ directory.')